# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrnetwork


class VerificationStatus(object):
    # Unset
    NONE = 0
    #
    PENDING = 1
    #
    VERIFIED = 2
    #
    FAILED = 3
    #
    EXPIRED = 4
