# automatically generated by the FlatBuffers compiler, do not modify

# namespace:

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class oid_t(object):
    __slots__ = ["_tab"]

    @classmethod
    def SizeOf(cls):
        return 16

    # oid_t
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # oid_t
    def Msb(self):
        return self._tab.Get(
            flatbuffers.number_types.Uint64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0)
        )

    # oid_t
    def Lsb(self):
        return self._tab.Get(
            flatbuffers.number_types.Uint64Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(8)
        )


def Createoid_t(builder, msb, lsb):
    builder.Prep(8, 16)
    builder.PrependUint64(lsb)
    builder.PrependUint64(msb)
    return builder.Offset()
