# automatically generated by the FlatBuffers compiler, do not modify

# namespace: log


class IpVersion(object):
    # Unset IP version.
    NONE = 0
    # IP version 4.
    IPV4 = 4
    # IP version 6.
    IPV6 = 6
