# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrnetwork


class VerificationType(object):
    # Unset
    NONE = 0
    # Member onboarded, using verification via member primary email.
    MEMBER_ONBOARD_EMAIL = 1
    # Member logged in, using verification via member primary email.
    MEMBER_LOGIN_EMAIL = 2
    # Member created a market, using verification via member primary email.
    MEMBER_CREATEMARKET_EMAIL = 3
    # Member joined a market, using verification via member primary email.
    MEMBER_JOINMARKET_EMAIL = 4
