# automatically generated by the FlatBuffers compiler, do not modify

# namespace: user

# CFC user roles (built into code, unconfigurable)
class UserRole(object):
    NONE = 0
    OWNER = 1
    ADMIN = 2
    USER = 3
    GUEST = 4
