# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbr

# XBR Market Actor type.
class ActorType(object):
    # Unset
    NONE = 0
    # Actor is a XBR Provider.
    PROVIDER = 1
    # Actor is a XBR Consumer.
    CONSUMER = 2
    # Actor is both a XBR Provider and XBR Consumer.
    PROVIDER_CONSUMER = 3
