# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrnetwork


class WalletType(object):
    # Unset
    NONE = 0
    # Account (primary) wallet was imported (the user provided the wallet public address).
    IMPORTED = 1
    # Account wallet was configured from Metamask.
    METAMASK = 2
    # Account wallet in hosted (in this database).
    HOSTED = 3
