# automatically generated by the FlatBuffers compiler, do not modify

# namespace: realmstore

# The serializer the app payload was serialized with before encryption.
class EncSerializer(object):
    # Unset
    NONE = 0
    # Payload was serialized in JSON
    JSON = 1
    # Payload was serialized in MsgPack
    MSGPACK = 2
    # Payload was serialized in CBOR
    CBOR = 3
    # Payload was serialized in UBJSON
    UBJSON = 4
    OPAQUE = 5
    FLATBUFFERS = 6
