# automatically generated by the FlatBuffers compiler, do not modify

# namespace: log

# Managed node worker types.
class MWorkerType(object):
    # Unset worker type.
    NONE = 0
    # Router worker.
    ROUTER = 1
    # Router worker.
    CONTAINER = 2
    # Router worker.
    GUEST = 3
    # Router worker.
    TESTEE = 4
    # Router worker.
    PROXY = 5
    # Router worker.
    XBRMM = 6
    # Router worker.
    HOSTMONITOR = 7
    # Router worker.
    CONTROLLER = 8
