# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrmm

# State of an (off-chain) transaction.
class TransactionState(object):
    # Unset
    NONE = 0
    # The (off-chain) transaction is currently in-flight.
    INFLIGHT = 1
    # The (off-chain) transaction has completed with error (it failed).
    FAILED = 2
    # The (off-chain) transaction has completed with success.
    SUCCESS = 3
