# automatically generated by the FlatBuffers compiler, do not modify

# namespace: log

# Managed nodes run-time (live) states.
class MNodeState(object):
    # Unset state.
    NONE = 0
    # Online and fully operational.
    ONLINE = 1
    # Online, but some error occurred and some features or entities might be unavailable.
    DEGRADED = 2
