# automatically generated by the FlatBuffers compiler, do not modify

# namespace: log

# CFC instance (master node) run-time (live) states.
class MRealmState(object):
    # Unset state.
    NONE = 0
    # Online and fully operational.
    ONLINE = 1
    # Online, but some error occurred and some features or entities might be unavailable.
    DEGRADED = 2
    # Ended gracefully.
    ENDED = 3
    # The state was determined as "not current" (eg when the previous running CFC was hard-killed) and the record was cleaned up (marked as ended).
    PURGED = 4
