# automatically generated by the FlatBuffers compiler, do not modify

# namespace: user

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


# CFC user roles on a management realm
class UserMrealmRoles(object):
    __slots__ = ["_tab"]

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = UserMrealmRoles()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsUserMrealmRoles(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)

    # UserMrealmRoles
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # List of roles the user has on the respective mrealm.
    # UserMrealmRoles
    def Roles(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(
                flatbuffers.number_types.Int8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1)
            )
        return 0

    # UserMrealmRoles
    def RolesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Int8Flags, o)
        return 0

    # UserMrealmRoles
    def RolesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # UserMrealmRoles
    def RolesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0


def UserMrealmRolesStart(builder):
    builder.StartObject(1)


def Start(builder):
    UserMrealmRolesStart(builder)


def UserMrealmRolesAddRoles(builder, roles):
    builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(roles), 0)


def AddRoles(builder, roles):
    UserMrealmRolesAddRoles(builder, roles)


def UserMrealmRolesStartRolesVector(builder, numElems):
    return builder.StartVector(1, numElems, 1)


def StartRolesVector(builder, numElems: int) -> int:
    return UserMrealmRolesStartRolesVector(builder, numElems)


def UserMrealmRolesEnd(builder):
    return builder.EndObject()


def End(builder):
    return UserMrealmRolesEnd(builder)
