# automatically generated by the FlatBuffers compiler, do not modify

# namespace: user

# CFC organization type
class OrganizationType(object):
    # No organization.
    NONE = 0
    # Personal or individual.
    PERSONAL = 1
    # Company or business.
    BUSINESS = 2
    # Public academics & teaching.
    ACADEMICS = 3
