# automatically generated by the FlatBuffers compiler, do not modify

# namespace: meta

# Documentation (text) formats.
class DocFormat(object):
    # Unset documentation format.
    NONE = 0
    # Plain text documentation format (default).
    TEXT = 1
    # Markdown text documentation format.
    MARKDOWN = 2
    # Restructered Text (ReST) text documentation format.
    REST = 3
    # Generic HTML.
    HTML = 4
    # Generic XML.
    XML = 5
    # DocBook XML
    DOCBOOK = 6
    # Jupyter notebook.
    NOTEBOOK = 7
