# automatically generated by the FlatBuffers compiler, do not modify

# namespace: mrealm

# Status of RouterWorkerGroups.
class WorkerGroupStatus(object):
    # Unset status.
    NONE = 0
    # Initial status of a newly created worker group.
    STOPPED = 1
    # The worker group is currently starting.
    STARTING = 2
    # The worker group is currently running.
    RUNNING = 3
    # The worker group is currently paused (it is serving, but the watch/change engine doesn't operate).
    PAUSED = 4
    # The worker group is currently stopping.
    STOPPING = 5
    # The worker group is stopped in an error state.
    ERROR = 6
    # The worker group is currently running, but degraded (not all components and services desired are actually running at this moment, but some have failed, but may be retried automatically).
    DEGRADED = 7
