# automatically generated by the FlatBuffers compiler, do not modify

# namespace: user

# User activation types (built into code, unconfigurable)
class ActivationType(object):
    NONE = 0
    REGISTRATION = 1
    LOGIN = 2
