# automatically generated by the FlatBuffers compiler, do not modify

# namespace: arealm

# WAMP URI check level applied in an application realm.
class UriCheckLevel(object):
    # No check level.
    NONE = 0
    # Check level "strict".
    STRICT = 1
    # Check level "loose".
    LOOSE = 2
