# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrnetwork

# XBR Account level.
class AccountLevel(object):
    # Unset
    NONE = 0
    # Account is active.
    ACTIVE = 1
    # Account is active and verified.
    VERIFIED = 2
    # Account is retired.
    RETIRED = 3
    # Account is subject to a temporary penalty.
    PENALTY = 4
    # Account is currently blocked and cannot current actively participate in the market.
    BLOCKED = 5
