# automatically generated by the FlatBuffers compiler, do not modify

# namespace: user

# User activation status (built into code, unconfigurable)
class ActivationStatus(object):
    NONE = 0
    PENDING = 1
    ACTIVE = 2
    EXPIRED = 3
