# automatically generated by the FlatBuffers compiler, do not modify

# namespace: realmstore

# The payload end-to-end encryption algorithm the payload is encrypted.
class EncAlgo(object):
    # Unset
    NONE = 0
    # WAMP-cryptobox end-to-end encrypted application payload
    CRYPTOBOX = 1
    # MQTT passthrough-mode application payload
    MQTT = 2
    # XBR end-to-end encrypted application payload
    XBR = 3
    OPAQUE = 4
