# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbr

# XBR Member level.
class MemberLevel(object):
    # Unset
    NONE = 0
    # Member is active.
    ACTIVE = 1
    # Member is active and verified.
    VERIFIED = 2
    # Member is retired.
    RETIRED = 3
    # Member is subject to a temporary penalty.
    PENALTY = 4
    # Member is currently blocked and cannot current actively participate in the market.
    BLOCKED = 5
