# automatically generated by the FlatBuffers compiler, do not modify

# namespace: xbrmm

# State of a XBR channel.
class ChannelState(object):
    # Unset
    NONE = 0
    # The channel is open on-chain, and now operating off-chain until closed.
    OPEN = 1
    # The off-chain channel is closing (one of the channel participants has requested to closed the channel)
    CLOSING = 2
    # The off-chain channel is closed and settled with no errors. The final channel state is committed on-chain.
    CLOSED = 3
    # The channel is closed with an error. The final channel state is committed on-chain.
    FAILED = 4
